package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Sets;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelPermissionMapper;
import com.syc.slm.slmbi.dto.CustomPermissionDTO;
import com.syc.slm.slmbi.dto.InstrumentPanelPermissionDTO;
import com.syc.slm.slmbi.entity.InstrumentPanel;
import com.syc.slm.slmbi.entity.InstrumentPanelConfig;
import com.syc.slm.slmbi.entity.InstrumentPanelPermission;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.InstrumentPanelPermissionService;
import com.syc.slm.slmbi.service.InstrumentPanelService;
import com.syc.slm.slmbi.service.InstrumentPanelTreeService;
import com.syc.slm.slmbi.vo.CustomPermissionVo;
import com.syc.slm.slmbi.vo.InstrumentPanelPermissionVo;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * slm-bi
 * 仪表板权限
 *
 * @author : lin
 * @date : 2021-03-25 19:04
 **/
@Service
@Slf4j
public class InstrumentPanelPermissionImpl
    extends ServiceImpl<InstrumentPanelPermissionMapper, InstrumentPanelPermission>
    implements InstrumentPanelPermissionService {

    @Autowired
    private InstrumentPanelService panelService;

    @Autowired
    private InstrumentPanelTreeService treeService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String savePermission(InstrumentPanelPermissionDTO permissions) {
        log.info("判断仪表板的名称是否重复");
        if (StringUtils.isBlank(permissions.getPanelName())) {
            throw new SysException("仪表板名称必填");
        }
        if (StringUtils.isBlank(permissions.getPanelId())) {
            throw new SysException("仪表板id必填,请联系管理员");
        }
        if (StringUtils.isBlank(permissions.getGroupId())) {
            throw new SysException("仪表板groupId必填,请联系管理员");
        }
        InstrumentPanel panel = panelService.getById(permissions.getPanelId());
        panel.setName(permissions.getPanelName());
        panel.setGroupId(permissions.getGroupId());
        panel.setAccessPermission(permissions.getAccessType());
        panel.setDescription(permissions.getDescription());

        log.info("删除原本的权限");
        delPermissionByPanelId(permissions.getPanelId(), PublishedStatus.WAIT_PUBLISHED.value);
        savePanelPermission(permissions.getCustomPermissions(),permissions.getPanelId(),permissions.getAccessType(),PublishedStatus.WAIT_PUBLISHED.value);
        
        log.info("更新仪表板的名称跟分组");
        treeService.updateFullNameByGroupId(permissions.getGroupId(),permissions.getPanelId(),permissions.getPanelName());
        panelService.updateById(panel);
        return permissions.getPanelId();
    }

    @Override
    public InstrumentPanelPermissionVo saveCopyOtherPermission(@NonNull String instrumentPanelId) {
        return selectPermissionByPanelId(instrumentPanelId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveCopyToOtherPermission(InstrumentPanelPermissionDTO permissions, List<String> otherInstrumentPanelIds) {
        if(CollectionUtils.isEmpty(otherInstrumentPanelIds)){
            throw new SysException("其他仪表板id必填，请联系管理员");
        }
        if(StringUtils.isBlank(permissions.getAccessType())){
            throw new SysException("访问权限类型必填");
        }
        List<InstrumentPanel> panels = panelService.getByIds(Sets.newHashSet(otherInstrumentPanelIds));
        if(CollectionUtils.isEmpty(panels)){
            throw new SysException("查找不到对应的仪表板,请联系管理员");
        }
        panels.forEach(panel->{
            panel.setAccessPermission(permissions.getAccessType());
            panel.setDescription(permissions.getDescription());
            log.info("删除原本的权限");
            delPermissionByPanelId(panel.getId(), PublishedStatus.WAIT_PUBLISHED.value);
            log.info("插入现有的权限");
            savePanelPermission(permissions.getCustomPermissions(), panel.getId(), panel.getAccessPermission(), PublishedStatus.WAIT_PUBLISHED.value);

            log.info("更新访问权限");
            panelService.updateById(panel);
        });

    }

    @Override
    @Transactional(readOnly = true)
    public InstrumentPanelPermissionVo selectPermissionByPanelId(@NonNull String instrumentPanelId) {
        InstrumentPanelPermissionVo vo = new InstrumentPanelPermissionVo();
        InstrumentPanel panel = panelService.getById(instrumentPanelId);
        vo.setAccessType(panel.getAccessPermission());
        vo.setDescription(panel.getDescription());
        List<CustomPermissionVo> vos = getPublishPermissionByPanelId(instrumentPanelId, PublishedStatus.WAIT_PUBLISHED.value);
        if(CollectionUtils.isEmpty(vos)){
            vos = getPublishPermissionByPanelId(instrumentPanelId, PublishedStatus.PUBLISHED.value);
        }
        vo.setCustomPermissions(vos);
        return vo;
    }

    @Override
    public void delPermissionByPanelId(@NonNull String panelId, String publishStatus) {
        List<InstrumentPanelPermission> permissions =getPermissionByPanelIdAndPublishStatus(panelId,publishStatus);
        if (CollectionUtils.isNotEmpty(permissions)) {
            permissions.forEach(x -> {
                x.setRecordStatus(SlmConstants.DATA_DELETED);
                baseMapper.updateById(x);
            });
        }
    }

    @Override
    @Transactional(readOnly = true)
    public List<CustomPermissionVo> getPublishPermissionByPanelId(@NonNull String panelId,
        @NonNull String publishStatus) {
        List<InstrumentPanelPermission> permissions = getPermissionByPanelIdAndPublishStatus(panelId,publishStatus);
        return CollectionUtils.isNotEmpty(permissions) ? SlmConstants.gson.fromJson(SlmConstants.gson.toJson(permissions), new TypeToken<List<CustomPermissionVo>>() {}.getType()) : null;
    }

    @Override
    public void savePanelPermission(List<CustomPermissionDTO> customPermissions,
        String panelId,
        String accessType,
        String publishStatus) {
        if (CollectionUtils.isNotEmpty(customPermissions)) {
            if (AccessType.CUSTOM.value.equals(accessType)) {
                if (CollectionUtils.isNotEmpty(customPermissions)) {
                    customPermissions.forEach(x -> {
                        InstrumentPanelPermission p = new InstrumentPanelPermission();
                        p.setId(IdWorker.get32UUID());
                        p.setBizId(x.getBizId());
                        p.setBizName(x.getBizName());
                        p.setBizType(x.getType());
                        p.setPanelId(panelId);
                        p.setPublished(publishStatus);
                        baseMapper.insert(p);
                    });
                }
            }
        }
    }

    @Override
    public void publishPermissionByPanelIds(List<String> ids) {
        LambdaQueryWrapper<InstrumentPanelPermission> where = new LambdaQueryWrapper<>();
        where.in(InstrumentPanelPermission::getPanelId, ids);
        where.eq(InstrumentPanelPermission::getRecordStatus, SlmConstants.DATA_VALID);
        List<InstrumentPanelPermission> permissions = baseMapper.selectList(where);
        Map<String, List<InstrumentPanelPermission>> permissionMaps = permissions.stream().collect(Collectors.groupingBy(InstrumentPanelPermission::getPanelId));
        if(CollectionUtils.isNotEmpty(permissionMaps)){
            permissionMaps.forEach((k,v)->{
                List<InstrumentPanelPermission> waitings =  permissions.stream().filter(x -> PublishedStatus.WAIT_PUBLISHED.value.equals(x.getPublished())).collect(Collectors.toList());
                List<InstrumentPanelPermission> publishs = permissions.stream().filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished())).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(waitings)) {
                    waitings.forEach(x -> {
                        x.setPublished(PublishedStatus.PUBLISHED.value);
                        baseMapper.updateById(x);
                    });
                    if (CollectionUtils.isNotEmpty(publishs)) {
                        publishs.forEach(x -> {
                            x.setRecordStatus(SlmConstants.DATA_DELETED);
                            baseMapper.updateById(x);
                        });
                    }
                }
            });
        }

    }

    @Override
    public List<InstrumentPanelPermission> getPermissionByPanelIdAndPublishStatus(String panelId, String publishStatus) {
        LambdaQueryWrapper<InstrumentPanelPermission> where = new LambdaQueryWrapper<>();
        where.eq(InstrumentPanelPermission::getPanelId, panelId);
        where.eq(InstrumentPanelPermission::getRecordStatus, SlmConstants.DATA_VALID);
        if(publishStatus!=null) {
            where.eq(InstrumentPanelPermission::getPublished, publishStatus);
        }
        return  baseMapper.selectList(where);
    }

    @Override
    public void offlinePermission(String panelId) {
        List<InstrumentPanelPermission> permissions = getPermissionByPanelIdAndPublishStatus(panelId,null);
        long waitP = permissions.stream().filter(x -> PublishedStatus.WAIT_PUBLISHED.value.equals(x.getPublished())).count();
        List<InstrumentPanelPermission> publish = permissions.stream().filter(x -> PublishedStatus.PUBLISHED.value.equals(x.getPublished())).collect(Collectors.toList());
        if(waitP>0){
            if(CollectionUtils.isNotEmpty(publish)){
                publish.forEach(x->{
                    x.setRecordStatus(SlmConstants.DATA_DELETED);
                    baseMapper.updateById(x);
                });
            }
        }else{
            if(CollectionUtils.isNotEmpty(publish)){
                publish.forEach(push->{
                    InstrumentPanelPermission p = new InstrumentPanelPermission();
                    BeanUtils.copyProperties(push, p);
                    p.setPublished(PublishedStatus.WAIT_PUBLISHED.value);
                    p.setId(IdWorker.get32UUID());
                    baseMapper.insert(p);
                    push.setRecordStatus(SlmConstants.DATA_DELETED);
                    baseMapper.updateById(push);
                });


            }
        }
    }
}
